/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author DNechita (dan.nechita@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 809
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "AD7780.h"

/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
#pragma config POSCMOD  = XT        // XT oscillator mode selected
#pragma config FNOSC    = PRIPLL    // Primary Oscillator with PLL Module
#pragma config FPLLMUL  = MUL_20    // 20x multiplier
#pragma config FPLLIDIV = DIV_2     // 2x divider
#pragma config FPLLODIV = DIV_1     // PLL output divided by 1
#pragma config FPBDIV   = DIV_1     // PBCLK is SYSCLK divided by 1
#pragma config FWDTEN   = OFF       // The WDT is not enabled

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/

/* List of available commands */
const char* commandsList[] = {"help?",
                              "power=",
                              "filter=",
                              "gain=",
                              "status=",
                              "rawdata?",
                              "voltage?",
                              "samples="};
const char* commandsDescription[] = {
"  -  Displays all available commands.",
"  -  Power off or on the device. Accepted values: 0, 1.",
"  -  Select the filter frequency. Accepted values: \r\n\
      0 - Filter frequency is 16.7 Hz \r\n\
      1 - Filter frequency is 10 Hz",
"  -  Select gain. Accepted values:\r\n\
      0 - Gain is set to 128(Maximum input voltage must be 37mV).\r\n\
      1 - Gain is set to 1",
"   - Shows the conversion status every ADC reading. Accepted values: 0, 1.",
"  -  Reads one sample from the selected channel.",
"  -  Reads one sample form the selected channel and converts it to voltage.",
"  -  Reads a number of samples of the current channel indicated by the user.\
Accepted values: 1 - 1024"};

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
double        commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converted value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    char  valSign  = 0;
    char  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    char  i = sizeof(localBuf) - 1;

    if(val < 0)
    {
        valSign = 1;
        val *= -1;
    }
    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > (sizeof(localBuf) - 4))
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    if(valSign == 1)
    {
        i --;
        localBuf[i] = '-';
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
    *buf = 0;
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{
    /* Variables holding information about the device */    
    long          adcValue         = 0;
    float         voltage          = 0;
    unsigned char gain             = 1;
    unsigned char conversionStatus = 0;
    unsigned char showStatus       = 0;
    /* Temporary variables */
    unsigned char  tempString[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    long           samples        = 0;

    /* Initialize the UART communication peripheral. */
    UART_Init(9600);
    /* Initialize the AD7328 device. */
    if(AD7780_Init())
    {
        CONSOLE_WriteString("AD7780 OK");
        UART_Write(0x0D);
        CONSOLE_WriteString("Available commands:");
        UART_Write(0x0D);
        for(displayCommand = 0; displayCommand < commandsNumber;
            displayCommand++)
        {
            CONSOLE_WriteString(commandsList[displayCommand]);
            CONSOLE_WriteString(commandsDescription[displayCommand]);
            UART_Write(0x0D);
        }
    }
    else
    {
        CONSOLE_WriteString("AD7780 Error");
        UART_Write(0x0D);
    }
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    // "help?" command
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))   // "power=" command
            {
                 if(commandParam == 1)
                 {
                    /* Power-up the device. */
                    AD7780_PDRST_HIGH;
                 }
                 else if(commandParam == 0)
                 {
                    /* Power-down the device. */
                    AD7780_PDRST_LOW;
                 }
                 else
                 {
                     CONSOLE_WriteString("Invalid parameter.");
                     UART_Write(0xD);
                     commandParam = 0;
                 }
                 /* Send feedback to user */
                 CONSOLE_WriteString(commandsList[command]);
                 itoa(tempString, (char)commandParam, 10);
                 CONSOLE_WriteString(tempString);
                 UART_Write(0xD);
            }
            if((command == 2) && (commandType != 0))   // "filter=" command
            {
                if(commandParam == 1)
                 {
                    AD7780_FILTER_HIGH;   // The update rate is set to 10 Hz.
                 }
                 else if(commandParam == 0)
                 {
                    AD7780_FILTER_LOW;   // The update rate is set to 16.7 Hz.
                 }
                 else
                 {
                     CONSOLE_WriteString("Invalid parameter.");
                     UART_Write(0xD);
                     commandParam = 0;
                 }
                 /* Send feedback to user */
                 CONSOLE_WriteString(commandsList[command]);
                 itoa(tempString, (char)commandParam, 10);
                 CONSOLE_WriteString(tempString);
                 UART_Write(0xD);
            }
            if((command == 3) && (commandType != 0))   // "gain=" command
            {
                if(commandParam == 1)
                 {
                    AD7780_GAIN_HIGH;   // Gain is set to 1.
                    gain = 1;
                 }
                 else if(commandParam == 0)
                 {
                    AD7780_GAIN_LOW;   // Gain is set to 128.
                    gain = 128;
                 }
                 else
                 {
                     CONSOLE_WriteString("Invalid parameter.");
                     UART_Write(0xD);
                     commandParam = 0;
                 }
                 /* Send feedback to user */
                 CONSOLE_WriteString(commandsList[command]);
                 itoa(tempString, (char)commandParam, 10);
                 CONSOLE_WriteString(tempString);
                 UART_Write(0xD);
            }
            if((command == 4) && (commandType != 0))   // "status=" command
            {
                if(commandParam == 1)
                {
                    showStatus = 1;
                }
                else if(commandParam == 0)
                {
                    showStatus = 0;
                }
                else
                {
                    CONSOLE_WriteString("Invalid parameter.");
                    UART_Write(0xD);
                    commandParam = 0;
                }
                /* Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                itoa(tempString, (char)commandParam, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 5) && (commandType != 0))   // "rawdata?" command
            {                
               /* Read one sample form the device. */
               adcValue = AD7780_ReadSample(&conversionStatus);
               /* Send the requested value to user */
               CONSOLE_WriteString("rawdata=");
               itoa(tempString, adcValue, 10);
               CONSOLE_WriteString(tempString);
               UART_Write(0xD);
               if(showStatus)
                {
                    /* Check gain flag. */
                    if(conversionStatus & AD7780_STAT_GAIN)
                    {
                        CONSOLE_WriteString("gain= 1");
                    }
                    else
                    {
                        CONSOLE_WriteString("gain= 128");
                    }
                    UART_Write(0xD);
                    /* Check filter flag. */
                    if(conversionStatus & AD7780_STAT_FILTER)
                    {
                        CONSOLE_WriteString("filter= 10 Hz");
                    }
                    else
                    {
                        CONSOLE_WriteString("filter= 16.7 Hz");
                    }
                    UART_Write(0xD);
                    /* Check for conversion error. */
                    if(conversionStatus & AD7780_STAT_ERR)
                    {
                        CONSOLE_WriteString("Conversion error occurred");
                    }
                    else
                    {
                        CONSOLE_WriteString("No conversion error.");
                    }
                    UART_Write(0xD);
                    /* Check for serial transfer error. */
                    if((conversionStatus &
                       (AD7780_STAT_PAT1 | AD7780_STAT_PAT0)) != 0x01)
                    {
                        CONSOLE_WriteString("Serial transfer error occurred.");
                    }
                    else
                    {
                        CONSOLE_WriteString("No serial transfer error.");
                    }
                    UART_Write(0xD);
                }
            }
            if((command == 6) && (commandType != 0))  // "voltage?" command
            {
               /* Read one sample form the device. */
               adcValue = AD7780_ReadSample(&conversionStatus);
               /* Convert raw data to voltage. */
               voltage = AD7780_ConvertToVoltage(adcValue, 3.3, gain);
               /* Send the requested value to user */
               CONSOLE_WriteString("voltage=");
               FloatToString(tempString, voltage);
               CONSOLE_WriteString(tempString);
               UART_Write(0xD);
            }
            if((command == 7) && (commandType != 0))  // "samples=" command
            {
                samples = commandParam;
                if((samples >= 1) && (samples <= 1024))
                {
                    while(samples--)
                    {
                        AD7780_WaitRdyGoLow();
                        /* Read one sample form the device. */
                        adcValue = AD7780_ReadSample(&conversionStatus);
                        /* Send the requested value to user */
                        itoa(tempString, adcValue, 10);
                        CONSOLE_WriteString(tempString);
                        UART_Write(0xD);
                    }
                }
                else
                {
                    CONSOLE_WriteString("Parameter out of range(1 - 1024).");
                }
            }
        }
        if(invalidCommand == commandsNumber)
        {
            /* Send feedback to user */
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    } // End while(1)
} // End main()
